git@vger.kernel.org mailing list mirror (one of many)
 help / color / mirror / code / Atom feed
From: "brian m. carlson" <sandals@crustytoothpaste.net>
To: <git@vger.kernel.org>
Cc: "SZEDER Gábor" <szeder.dev@gmail.com>,
	"Taylor Blau" <me@ttaylorr.com>, "Jeff King" <peff@peff.net>,
	"Junio C Hamano" <gitster@pobox.com>
Subject: [PATCH v4 1/2] path: add a function to check for path suffix
Date: Sun, 18 Aug 2019 18:44:02 +0000	[thread overview]
Message-ID: <20190818184403.861907-2-sandals@crustytoothpaste.net> (raw)
In-Reply-To: <20190818184403.861907-1-sandals@crustytoothpaste.net>

We have a function to strip the path suffix from a commit, but we don't
have one to check for a path suffix. For a plain filename, we can use
basename, but that requires an allocation, since POSIX allows it to
modify its argument. Refactor strip_path_suffix into a helper function
and a new function, ends_with_path_components, to meet this need.

Signed-off-by: brian m. carlson <sandals@crustytoothpaste.net>
---
 path.c | 39 ++++++++++++++++++++++++++++++---------
 path.h |  3 +++
 2 files changed, 33 insertions(+), 9 deletions(-)

diff --git a/path.h b/path.h
index 2ba6ca58c8..14d6dcad16 100644
--- a/path.h
+++ b/path.h
@@ -193,4 +193,7 @@ const char *git_path_merge_head(struct repository *r);
 const char *git_path_fetch_head(struct repository *r);
 const char *git_path_shallow(struct repository *r);
 
+
+int ends_with_path_components(const char *path, const char *components);
+
 #endif /* PATH_H */

  reply	other threads:[~2019-08-18 18:44 UTC|newest]

Thread overview: 41+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2019-08-09 10:02 [PATCH] apply: reload .gitattributes after patching it brian m. carlson
2019-08-09 11:14 ` Taylor Blau
2019-08-09 11:25   ` brian m. carlson
2019-08-09 11:36     ` Jeff King
2019-08-09 11:47       ` brian m. carlson
2019-08-09 12:43       ` SZEDER Gábor
2019-08-09 13:51         ` Jeff King
2019-08-09 22:08           ` brian m. carlson
2019-08-11 17:47 ` [PATCH v2 0/2] Honor .gitattributes with rebase --am brian m. carlson
2019-08-11 17:47   ` [PATCH v2 1/2] path: add a function to check for path suffix brian m. carlson
2019-08-12  0:32     ` Junio C Hamano
2019-08-12  1:10       ` brian m. carlson
2019-08-12  4:36         ` SZEDER Gábor
2019-08-12 16:49         ` Junio C Hamano
2019-08-12 22:40           ` brian m. carlson
2019-08-13  1:13             ` Jeff King
2019-08-13 16:40             ` Junio C Hamano
2019-08-13  6:36           ` SZEDER Gábor
2019-08-13 16:42             ` Junio C Hamano
2019-08-11 17:47   ` [PATCH v2 2/2] apply: reload .gitattributes after patching it brian m. carlson
2019-08-13  2:43 ` [PATCH v3 0/2] Honor .gitattributes with rebase --am brian m. carlson
2019-08-13  2:43   ` [PATCH v3 1/2] path: add a function to check for path suffix brian m. carlson
2019-08-13  2:43   ` [PATCH v3 2/2] apply: reload .gitattributes after patching it brian m. carlson
2019-08-13 18:08     ` Junio C Hamano
2019-08-15 22:10       ` Junio C Hamano
2019-08-16  0:22         ` brian m. carlson
2019-08-13 22:37     ` Junio C Hamano
2019-08-13 22:48       ` brian m. carlson
2019-08-18 18:44 ` [PATCH v4 0/2] Honor .gitattributes with rebase --am brian m. carlson
2019-08-18 18:44   ` brian m. carlson [this message]
2019-08-18 18:44   ` [PATCH v4 2/2] apply: reload .gitattributes after patching it brian m. carlson
2019-08-19  9:41     ` Phillip Wood
2019-08-19  9:55       ` Phillip Wood
2019-08-20  3:05         ` brian m. carlson
2019-08-20  8:56           ` Phillip Wood
2019-08-20  2:45       ` brian m. carlson
2019-08-20  8:52         ` Phillip Wood
2019-08-20 18:24           ` Junio C Hamano
2019-08-20 18:32             ` Phillip Wood
2019-08-26 15:09               ` Phillip Wood
2019-08-26 16:41                 ` Junio C Hamano

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

  List information: http://vger.kernel.org/majordomo-info.html

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20190818184403.861907-2-sandals@crustytoothpaste.net \
    --to=sandals@crustytoothpaste.net \
    --cc=git@vger.kernel.org \
    --cc=gitster@pobox.com \
    --cc=me@ttaylorr.com \
    --cc=peff@peff.net \
    --cc=szeder.dev@gmail.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://80x24.org/mirrors/git.git

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).
diff --git a/path.c b/path.c
index 25e97b8c3f..e3da1f3c4e 100644
--- a/path.c
+++ b/path.c
@@ -1221,31 +1221,52 @@ static inline int chomp_trailing_dir_sep(const char *path, int len)
 }
 
 /*
  * If path ends with suffix (complete path components), returns the
  * part before suffix (sans trailing directory separators).
  * Otherwise returns NULL.
  */
-char *strip_path_suffix(const char *path, const char *suffix)
+static ssize_t stripped_path_suffix_offset(const char *path, const char *suffix)
 {
 	int path_len = strlen(path), suffix_len = strlen(suffix);
 
 	while (suffix_len) {
 		if (!path_len)
-			return NULL;
+			return -1;
 
 		if (is_dir_sep(path[path_len - 1])) {
 			if (!is_dir_sep(suffix[suffix_len - 1]))
-				return NULL;
+				return -1;
 			path_len = chomp_trailing_dir_sep(path, path_len);
 			suffix_len = chomp_trailing_dir_sep(suffix, suffix_len);
 		}
 		else if (path[--path_len] != suffix[--suffix_len])
-			return NULL;
+			return -1;
 	}
 
 	if (path_len && !is_dir_sep(path[path_len - 1]))
-		return NULL;
-	return xstrndup(path, chomp_trailing_dir_sep(path, path_len));
+		return -1;
+	return chomp_trailing_dir_sep(path, path_len);
+}
+
+/*
+ * Returns true if the path ends with components, considering only complete path
+ * components, and false otherwise.
+ */
+int ends_with_path_components(const char *path, const char *components)
+{
+	return stripped_path_suffix_offset(path, components) != -1;
+}
+
+/*
+ * If path ends with suffix (complete path components), returns the
+ * part before suffix (sans trailing directory separators).
+ * Otherwise returns NULL.
+ */
+char *strip_path_suffix(const char *path, const char *suffix)
+{
+	ssize_t offset = stripped_path_suffix_offset(path, suffix);
+
+	return offset == -1 ? NULL : xstrndup(path, offset);
 }
 
 int daemon_avoid_alias(const char *p)
@@ -1221,31 +1221,52 @@ static inline int chomp_trailing_dir_sep(const char *path, int len)
 
 /*
- * If path ends with suffix (complete path components), returns the
- * part before suffix (sans trailing directory separators).
- * Otherwise returns NULL.
+ * If path ends with suffix (complete path components), returns the offset of
+ * the last character in the path before the suffix (sans trailing directory
+ * separators), and -1 otherwise.
  */